const { declare } = require('@babel/helper-plugin-utils');

const defaultTargets = {};

function buildTargets({ additionalTargets }) {
  return { ...defaultTargets, ...additionalTargets };
}

module.exports = declare((api, options) => {
  // see docs about api at https://babeljs.io/docs/en/config-files#apicache
  api.assertVersion('^7.0.0');

  const { modules, targets = buildTargets(options) } = options;

  if (typeof modules !== 'undefined' && typeof modules !== 'boolean' && modules !== 'auto') {
    throw new TypeError(
      '@nuofe/babel-preset-ndk only accepts `true`, `false`, or `"auto"` as the value of the "modules" option',
    );
  }

  const computedModulesOption = modules === false ? false : 'auto';

  const debug = typeof options.debug === 'boolean' ? options.debug : false;

  return {
    presets: [
      [
        require('@babel/preset-env'),
        {
          debug,
          modules: computedModulesOption,
          targets,
          useBuiltIns: 'usage',
          corejs: 2,
        },
      ],
    ],
    plugins: [
      [
        require('@babel/plugin-transform-modules-commonjs'),
        {
          loose: true,
        },
      ],
    ],
  };
});
